package com.alibaba.cloud.acm;

import com.alibaba.edas.acm.ConfigService;
import com.alibaba.edas.acm.exception.ConfigException;
import com.alibaba.edas.acm.listener.ConfigChangeListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.StringReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

/**
 * @author hexu.hxy
 * @date 2018/5/22 示例代码，仅用于示例测试
 */
public class SampleApp {
	private static final Logger LOGGER = LoggerFactory.getLogger(SampleApp.class);
	private volatile Properties properties;

	public static void main(String[] args) throws ConfigException {
		SampleApp sampleApp = new SampleApp();
		sampleApp.init();
		int count = sampleApp.count();
		System.out.println("总记录数: " + count);
	}

	private void init() throws ConfigException {
		initACM();
		initJDBCProperties();
	}

	private void initACM() {
		Properties properties = new Properties();
		String namespace = "1ca01ca0-11b0-1e01-0df1-d1010101bc10";
		String endpoint = "acm.aliyun.com";
		properties.put("endpoint", endpoint);
		properties.put("namespace", namespace);
		ConfigService.init(properties);
	}

	private void initJDBCProperties() throws ConfigException {
		String dataId = "com.alibaba.cloud.acm:jdbc-sample.properties";
		String group = "DEFAULT_GROUP";
		properties = ConfigService.getConfig2Properties(dataId, group, 3000);
		// 添加配置监听
		ConfigService.addListener(dataId, group, new ConfigChangeListener() {
			@Override
			public void receiveConfigInfo(String content) {
				Properties properties = new Properties();
				try {
					properties.load(new StringReader(content));
				} catch (IOException e) {
					LOGGER.error("addListener", e);
				}
				SampleApp.this.properties = properties;
			}
		});
	}

	private int count() {
		String url = properties.getProperty("jdbc.url");
		String username = properties.getProperty("jdbc.username");
		String password = properties.getProperty("jdbc.password");
		Connection connection = null;
		Statement statement = null;
		ResultSet resultSet = null;
		try {
			connection = DriverManager.getConnection(url, username, password);
			statement = connection.createStatement();
			resultSet = statement.executeQuery("select count(*) from `user`");
			if (resultSet.next()) {
				return resultSet.getInt(1);
			}
		} catch (SQLException e) {
			LOGGER.error("count", e);
		} finally {
			close(connection, statement, resultSet);
		}
		return 0;
	}

	private void close(Connection connection, Statement statement, ResultSet resultSet) {
		if (resultSet != null) {
			try {
				resultSet.close();
			} catch (SQLException e) {
				LOGGER.error("close resultSet", e);
			}
		}
		if (statement != null) {
			try {
				statement.close();
			} catch (SQLException e) {
				LOGGER.error("close statement", e);
			}
		}
		if (connection != null) {
			try {
				connection.close();
			} catch (SQLException e) {
				LOGGER.error("close connection", e);
			}
		}
	}
}
